@@ -48,6 +48,7 @@
 
 #ifdef _MSWINDOWS_
 #include <direct.h>
+#include <winioctl.h>
 #endif
 
 #ifdef  HAVE_SYS_INOTIFY_H
@@ -64,6 +65,21 @@ const fsys::offset_t fsys::end = (size_t)(-1);
 
 #ifdef  _MSWINDOWS_
 
+// removed from some sdk versions...
+struct LOCAL_REPARSE_DATA_BUFFER
+{
+    DWORD  ReparseTag;
+    WORD   ReparseDataLength;
+    WORD   Reserved;
+    
+    // IO_REPARSE_TAG_MOUNT_POINT specifics follow
+    WORD   SubstituteNameOffset;
+    WORD   SubstituteNameLength;
+    WORD   PrintNameOffset;
+    WORD   PrintNameLength;
+    WCHAR  PathBuffer[1];    
+};
+
 int fsys::remapError(void)
 {
     DWORD err = GetLastError();
@@ -335,6 +351,20 @@ void fsys::create(const char *path, access_t access, unsigned mode)
     DWORD smode = 0;
     DWORD attr = FILE_ATTRIBUTE_NORMAL;
     unsigned flags = 0;
+
+    const char *cp = strrchr(path, '\\');
+    const char *cp2 = strrchr(path, '/');
+    if(cp2 > cp)
+        cp = cp2;
+
+    if(!cp)
+        cp = path;
+    else
+        ++cp;
+
+    if(*cp == '.')
+        attr = FILE_ATTRIBUTE_HIDDEN;
+
     switch(access)
     {
     case ACCESS_RDONLY:
@@ -786,6 +816,117 @@ fsys::~fsys()
     close();
 }
 
+int fsys::linkinfo(const char *path, char *buffer, size_t size)
+{
+#if defined(_MSWINDOWS_)
+    HANDLE h;
+    char reparse[MAXIMUM_REPARSE_DATA_BUFFER_SIZE];
+    DWORD rsize;
+
+    if(!fsys::islink(path))
+        return EINVAL;
+
+    h = CreateFile(path, GENERIC_READ, 0, 0, OPEN_EXISTING,
+        FILE_FLAG_BACKUP_SEMANTICS | FILE_FLAG_OPEN_REPARSE_POINT, 0);
+
+    if(!h || h == INVALID_HANDLE_VALUE)
+        return EINVAL;
+
+    memset(reparse, 0, sizeof(reparse));
+    LOCAL_REPARSE_DATA_BUFFER *rb = (LOCAL_REPARSE_DATA_BUFFER*)&reparse;
+
+    if(!DeviceIoControl(h, FSCTL_GET_REPARSE_POINT, NULL, 0, (LPVOID *)rb, MAXIMUM_REPARSE_DATA_BUFFER_SIZE, &rsize, 0)) {
+        CloseHandle(h);
+        return remapError();
+    }
+
+#ifdef  UNICODE
+    String::set(buffer, size, rb.PathBuffer);
+#else
+    WideCharToMultiByte(CP_THREAD_ACP, 0, rb->PathBuffer, rb->SubstituteNameLength / sizeof(WCHAR) + 1, buffer, size, "", FALSE);
+#endif
+    CloseHandle(h);
+    return 0;
+#elif defined(HAVE_READLINK)
+    if(::readlink(path, buffer, size))
+        return remapError();
+    return 0;
+#else
+    return EINVAL;
+#endif
+}
+
+int fsys::link(const char *path, const char *target)
+{
+#if defined(_MSWINDOWS_)
+    TCHAR dest[512];
+    HANDLE h;
+    char reparse[MAXIMUM_REPARSE_DATA_BUFFER_SIZE];
+    char *part;
+    DWORD size;
+    WORD len;
+
+    lstrcpy(dest, "\\??\\");
+    if(!GetFullPathName(path, sizeof(dest) - (4 * sizeof(TCHAR)), &dest[4], &part) || GetFileAttributes(&dest[4]) == -1)
+        return remapError();
+
+    memset(reparse, 0, sizeof(reparse));
+    LOCAL_REPARSE_DATA_BUFFER *rb = (LOCAL_REPARSE_DATA_BUFFER*)&reparse;
+    
+    if(!MultiByteToWideChar(CP_THREAD_ACP, MB_PRECOMPOSED, dest, lstrlenA(dest) + 1, rb->PathBuffer, lstrlenA(dest) + 1))
+        return remapError();
+    
+    len = lstrlenW(rb->PathBuffer) * 2;
+    rb->ReparseTag = IO_REPARSE_TAG_MOUNT_POINT;
+    rb->ReparseDataLength = len + 12;
+    rb->SubstituteNameLength = len;
+    rb->PrintNameOffset = len + 2;
+    h = CreateFile(target, GENERIC_READ | GENERIC_WRITE, 0, 0, OPEN_EXISTING,
+        FILE_FLAG_BACKUP_SEMANTICS | FILE_FLAG_OPEN_REPARSE_POINT, 0);
+    if(!h || h == INVALID_HANDLE_VALUE)
+        return remapError();
+    if(!DeviceIoControl(h, FSCTL_SET_REPARSE_POINT, (LPVOID)rb, rb->ReparseDataLength + FIELD_OFFSET(LOCAL_REPARSE_DATA_BUFFER, SubstituteNameOffset), NULL, 0, &size, 0)) {
+        CloseHandle(h);
+        return remapError();
+    }
+    CloseHandle(h);
+    return 0;
+#elif defined(HAVE_SYMLINK)
+    if(::symlink(path, target))
+        return remapError();
+    return 0;
+#else
+    if(::link(path, target))
+        return remapError();
+    return 0;
+#endif
+}
+
+int fsys::unlink(const char *path)
+{
+#ifdef  _MSWINDOWS_
+    HANDLE h = INVALID_HANDLE_VALUE;
+    if(islink(path))
+        h = CreateFile(path, GENERIC_READ | GENERIC_WRITE, 0, 0, OPEN_EXISTING,
+            FILE_FLAG_BACKUP_SEMANTICS | FILE_FLAG_OPEN_REPARSE_POINT, 0);
+    if(!h || h != INVALID_HANDLE_VALUE) {
+        REPARSE_GUID_DATA_BUFFER rb;
+        memset(&rb, 0, sizeof(rb));
+        DWORD size;
+        rb.ReparseTag = IO_REPARSE_TAG_MOUNT_POINT;
+        if(!DeviceIoControl(h, FSCTL_DELETE_REPARSE_POINT, &rb,
+            REPARSE_GUID_DATA_BUFFER_HEADER_SIZE, NULL, 0, &size, 0)) {
+            CloseHandle(h);
+            return remapError();
+        }
+        CloseHandle(h);
+        ::remove(path);
+        return 0;
+    }
+#endif
+    return fsys::remove(path);
+}
+
 int fsys::remove(const char *path)
 {
     if(::remove(path))
@@ -847,7 +988,12 @@ bool fsys::isfile(const char *path)
 
 bool fsys::islink(const char *path)
 {
-#ifdef  HAVE_LSTAT
+#if defined(_MSWINDOWS_)
+    DWORD attr = GetFileAttributes(path);
+    if (attr == 0xffffffff || !(attr & FILE_ATTRIBUTE_REPARSE_POINT))
+        return false;
+    return true;
+#elif defined(HAVE_LSTAT)
     struct stat ino;
 
     if(::lstat(path, &ino))
@@ -1052,6 +1198,28 @@ void *fsys::find(mem_t addr, const char *sym)
 
 #endif
 
+bool fsys::ishidden(const char *path)
+{
+#ifdef  _MSWINDOWS_
+    DWORD attr = GetFileAttributes(path);
+    if(attr == (DWORD)~0l)
+        return false;
+
+    return ((attr & FILE_ATTRIBUTE_HIDDEN) != 0);
+#else
+    const char *cp = strrchr(path, '/');
+    if(cp)
+        ++cp;
+    else
+        cp = path;
+
+    if(*cp == '.')
+        return true;
+
+    return false;
+#endif
+}
+
 charfile::charfile(const char *file, const char *mode)
 {
     fp = NULL;
@@ -1194,3 +1362,17 @@ String str(charfile& so, strsize_t size)
     return s;
 }
 
+fsys::fsys(fd_t handle)
+{
+    fd = handle;
+    ptr = NULL;
+    error = 0;
+}
+
+void fsys::set(fd_t handle)
+{
+    close();
+    fd = handle;
+    ptr = NULL;
+    error = 0;
+}
