@@ -1214,6 +1214,97 @@ tail:
         String::add(buf, size, ".exe");
 }
 
+void shell::relocate(const char *argv0)
+{
+}
+
+String shell::path(path_t id)
+{
+    char buf[512];
+
+    string_t result = "";
+
+    if(!_domain)
+        return result;
+
+    switch(id) {
+    case SYSTEM_CERTIFICATES:
+        result ^= SSL_CERTS;
+        break;
+    case SYSTEM_KEYS:
+        result ^= SSL_PRIVATE;
+        break;
+    case PROGRAM_CONFIG:
+        result = str("~\\Software\\Applications\\") + _domain;
+        break;
+    case SERVICE_CONFIG:
+        result = str("-\\SOFTWARE\\Services\\") + _domain;
+        break;
+    case USER_DEFAULTS:
+        if(GetEnvironmentVariable("SystemRoot", buf, sizeof(buf)))
+            result = str(buf) + "\\" + _domain + ".ini";
+        break;
+    case USER_HOME:
+        if(GetEnvironmentVariable("USERPROFILE", buf, sizeof(buf)))
+            result = str(buf);
+        break;
+    case SERVICE_DATA:
+    case USER_DATA:
+        if(GetEnvironmentVariable("APPDATA", buf, sizeof(buf))) {
+            result = str(buf) + "\\" + _domain;
+            fsys::createDir(*result, 0700);
+        }
+        break;
+    case USER_CONFIG:
+        if(GetEnvironmentVariable("USERPROFILE", buf, sizeof(buf))) {
+            result = str(buf) + "\\Local Settings\\" + _domain;
+            fsys::createDir(*result, 0700);
+        }
+        break;
+    case USER_CACHE:
+    case SERVICE_CACHE:
+        if(GetEnvironmentVariable("TEMP", buf, sizeof(buf))) {
+            result = str(buf) + "\\" + _domain;
+            fsys::createDir(*result, 0700);
+            break;
+        }
+        if(GetEnvironmentVariable("APPDATA", buf, sizeof(buf))) {
+            result = str(buf) + "\\" + _domain;
+            fsys::createDir(*result, 0700);
+        }
+        break;
+    case SYSTEM_TEMP:
+        fsys::createDir("c:\\temp", 0777);
+        result ^= "c:\\temp";
+        break;
+    case PROGRAM_TEMP:
+        snprintf(buf, sizeof(buf), "$$%ld$$.tmp", (long)GetCurrentProcessId());
+        result = str("c:\\temp") + str(buf);
+        break;
+    case SYSTEM_ETC:
+        if(GetEnvironmentVariable("SystemRoot", buf, sizeof(buf)))
+            result = str(buf) + "\\etc";
+        break;
+    case SYSTEM_CFG:
+        result ^= UCOMMON_CFGPATH;
+        break;
+    case SYSTEM_VAR:
+        result ^= UCOMMON_VARPATH;
+        break;
+    case SYSTEM_PREFIX:
+        result ^= UCOMMON_PREFIX;
+        break;
+    case SYSTEM_SHARE:
+        result ^= UCOMMON_PREFIX;
+        break;
+    case PROGRAM_PLUGINS:
+        result = str(UCOMMON_PREFIX) + "\\plugins\\" + _domain;
+        break;
+    }
+
+    return result;
+}
+
 const char *shell::getenv(const char *id, const char *value)
 {
     char buf[512];
@@ -1546,6 +1637,142 @@ exit:
 
 #else
 
+static const char *system_prefix = UCOMMON_PREFIX;
+
+void shell::relocate(const char *argv0)
+{
+#ifdef  HAVE_REALPATH
+    char *path0 = realpath(argv0, NULL);
+    if(!path0)
+        return;
+
+    // strip out exe name...
+    char *cp = strrchr(path0, '/');
+    if(cp) {
+        *cp = 0;
+        // strip out bin subdir...
+        cp = strrchr(path0, '/');
+        if(cp && (eq(cp, "/bin") || eq(cp, "/sbin"))) {
+            *cp = 0;
+            system_prefix = path0;
+        }
+    }
+#endif
+}
+
+String shell::path(path_t id)
+{
+    string_t result = "";
+    const char *home = NULL;
+    char buf[65];
+
+    if(!_domain)
+        return result;
+
+    switch(id) {
+    case SYSTEM_CERTIFICATES:
+        result = str(SSL_CERTS);
+        break;
+    case SYSTEM_KEYS:
+        result = str(SSL_PRIVATE);
+        break;
+    case USER_DEFAULTS:
+        home = ::getenv("HOME");
+        if(!home)
+            break;
+        result = str(home) + "/." + _domain + "rc";
+        break;
+    case USER_HOME:
+        home = ::getenv("HOME");
+        if(!home)
+            break;
+        result = str(home);
+        break;
+    case SERVICE_DATA:
+        result = str(UCOMMON_VARPATH "/lib/") + _domain;
+        break;
+    case USER_DATA:
+        home = ::getenv("HOME");
+        if(!home)
+            break;
+#ifdef  __APPLE__
+        result = str(home) + "/Library/Application Support/" + _domain;
+#else
+        result = str(home) + "/.local/share/" + _domain;
+#endif
+        break;
+    case USER_CONFIG:
+        home = ::getenv("HOME");
+        if(!home)
+            break;
+#ifdef  __APPLE__
+        result = str(home) + "/Library/Preferences/" + _domain;
+#else
+        result = str(home) + "/.config/" + _domain;
+#endif
+        fsys::createDir(*result, 0700);
+        break;
+    case USER_CACHE:
+        home = ::getenv("HOME");
+        if(!home)
+            break;
+#ifdef  __APPLE__
+        result = str(home) + "/Library/Caches/" + _domain;
+#else
+        result = str(home) + "/.cache/" + _domain;
+#endif
+        break;
+    case SERVICE_CACHE:
+        result = str(UCOMMON_VARPATH "/cache/") + _domain;
+        break;
+    case PROGRAM_CONFIG:
+        home = ::getenv("HOME");
+        if(!home)
+            break;
+#ifdef  __APPLE__
+        result = str(home) + "/Library/Preferences/" + _domain;
+#else
+        result = str(home) + "/.config/" + _domain;
+#endif
+        fsys::createDir(*result, 0700);
+
+#ifdef  __APPLE__
+        result = result + "/" + _domain + ".conf";
+#else
+        result = result + "/" + _domain + "rc";
+#endif
+        break;
+    case SERVICE_CONFIG:
+        result = str(UCOMMON_CFGPATH) + "/" + _domain + ".conf";
+        break;
+    case SYSTEM_TEMP:
+        result ^= "/tmp";
+        break;
+    case PROGRAM_TEMP:
+        snprintf(buf, sizeof(buf), ".$$%ld$$.tmp", (long)getpid());
+        result = str("/tmp") + str(buf);
+        break;
+    case SYSTEM_ETC:
+    case SYSTEM_CFG:
+        result ^= UCOMMON_CFGPATH;
+        break;
+    case SYSTEM_VAR:
+        result ^= UCOMMON_VARPATH;
+        break;
+    case SYSTEM_PREFIX:
+        result ^= system_prefix;
+        break;
+    case SYSTEM_SHARE:
+        result = str(system_prefix) + "/share";
+        break;
+    case PROGRAM_PLUGINS:
+        result = str(system_prefix) + "/lib/" + _domain;
+        break;
+    }
+
+    return result;
+}
+
 const char *shell::getenv(const char *id, const char *value)
 {
     const char *v = ::getenv(id);
@@ -1924,28 +2151,19 @@ const char *shell::text(const char *msg)
 
 void shell::bind(const char *name)
 {
-#ifdef  _MSWINDOWS_
-    const char *locale = "C:\\Program Files\\Common Files\\locale";
+    string_t locale;
+    const char *prior = _domain;
 
-    if(fsys::isdir("locale")) {
-        char prefix[256];
-        fsys::getPrefix(prefix, sizeof(prefix));
-        String::add(prefix, sizeof(prefix), "/locale");
-        locale = strdup(prefix);
-    }
-
-#else
-    const char *locale = UCOMMON_LOCALE;
-#endif
-
-    if(!_domain) {
-        setlocale(LC_ALL, "");
-        bindtextdomain("ucommon", locale);
-    }
-
-    bindtextdomain(name, locale);
-    textdomain(name);
     _domain = name;
+    locale = path(SYSTEM_SHARE) + str("/locale");
+
+    if(!prior) {
+        setlocale(LC_ALL, "");
+        bindtextdomain("ucommon", *locale);
+    }
+
+    bindtextdomain(name, *locale);
+    textdomain(name);
 }
 
 void shell::rebind(const char *name)
@@ -2303,4 +2521,13 @@ void shell::setsym(const char *name, const char *value)
     symlock.release();
 }
 
+String shell::path(path_t id, const char *dir)
+{
+    string_t result;
 
+    if(*dir == '\\' || *dir == '/')
+        result = dir;
+    else
+        result = path(id) + "/" + dir;
+    return result;
+}
