@@ -135,6 +135,8 @@ protected:
     int     error;
 
 public:
+    typedef struct stat fileinfo_t;
+
 #ifdef  _MSWINDOWS_
     static int remapError(void);
 #else
@@ -172,6 +174,11 @@ public:
      */
     fsys();
 
+    /**
+     * Contruct fsys from raw file handle.
+     */
+    fsys(fd_t handle);
+
     /**
      * Copy (dup) an existing fsys descriptor.
      * @param descriptor to copy from.
@@ -245,6 +252,12 @@ public:
     inline fd_t getHandle(void) const
         {return fd;};
 
+    /**
+     * Set with external descriptor.  Closes existing file if open.
+     * @param descriptor of open file.
+     */
+    void set(fd_t descriptor);
+
     /**
      * Set the position of a file descriptor.
      * @param offset from start of file or "end" to append.
@@ -282,6 +295,9 @@ public:
      */
     int stat(struct stat *buffer);
 
+    inline int fileinfo(fileinfo_t *buffer)
+        {return stat(buffer);};
+
     /**
      * Truncate file to specified length.  The file pointer is positioned
      * to the new end of file.
@@ -319,6 +335,9 @@ public:
      */
     static int stat(const char *path, struct stat *buffer);
 
+    static inline int fileinfo(const char *path, fileinfo_t *buffer)
+        {return stat(path, buffer);};
+
     /**
      * Remove a file.
      * @param path of file.
@@ -371,6 +390,13 @@ public:
      */
     static bool islink(const char *path);
 
+    /**
+     * Test if path is a hidden file.
+     * @param path to test.
+     * @return true if exists and is hidden.
+     */
+    static bool ishidden(const char *path);
+
     /**
      * Read data from file descriptor or directory.
      * @param descriptor to read from.
@@ -454,6 +480,31 @@ public:
      */
     static int removeDir(const char *path);
 
+    /**
+     * Remove a symbolic link explicitly.  Other kinds of files are also
+     * deleted.  This should be used when uncertain about symlinks requiring
+     * special support.
+     * @param path to remove.
+     * @return error number or 0 on success.
+     */
+    static int unlink(const char *path);
+
+    /**
+     * Create a symbolic link.
+     * @param path to create.
+     * @param target of link.
+     * @return error number or 0 on success.
+     */
+    static int link(const char *path, const char *target);
+
+    /**
+     * Read a symbolic link to get it's target.
+     * @param path of link.
+     * @param buffer to save target into.
+     * @param size of buffer.
+     */
+    static int linkinfo(const char *path, char *buffer, size_t size);
+
     /**
      * Close a file descriptor or directory directly.
      * @param descriptor to close.
