@@ -104,6 +104,16 @@ public:
      */
     typedef enum {FAIL = 0, ERR, WARN, NOTIFY, INFO, DEBUG0} loglevel_t;
 
+    /**
+     * Path types to retrieve.
+     */
+    typedef enum {
+        PROGRAM_CONFIG, SERVICE_CONFIG, USER_DEFAULTS, SYSTEM_CERTIFICATES,
+        SYSTEM_KEYS, USER_HOME, SERVICE_DATA, SYSTEM_TEMP, USER_CACHE,
+        SERVICE_CACHE, USER_DATA, USER_CONFIG, SYSTEM_CFG, SYSTEM_ETC,
+        SYSTEM_VAR, SYSTEM_PREFIX, SYSTEM_SHARE, PROGRAM_PLUGINS,
+        PROGRAM_TEMP} path_t;
+
     /**
      * Log process handler.
      */
@@ -589,6 +599,27 @@ public:
      */
     static int systemf(const char *format, ...) __PRINTF(1,2);
 
+    /**
+     * Set relative prefix.  Used for OS/X relocatable applications.
+     * @param argv0 path of executable.
+     */
+    static void relocate(const char *argv0);
+
+    /**
+     * Get a system path.  This is used to get directories for application
+     * specific data stores and default paths for config keys.
+     * @param id of path to return.
+     * @return path string or emptry string if not supported.
+     */
+    static String path(path_t id);
+
+    /**
+     * Get a merged path.  If the path requested is a full path, then
+     * the prefix is ignored.
+     * @param directory path to merge with prefix.
+     */
+    static String path(path_t id, const char *directory);
+
     /**
      * Bind application to text domain for internationalization.  This
      * is useful if the argv0 argument can vary because of a symlinked
